From 742c615520e94baa0dba51d15fc4a46391a57330 Mon Sep 17 00:00:00 2001
From: Mark Crichton <crichton@tardis.(none)>
Date: Tue, 14 Sep 2010 22:16:16 -0400
Subject: [PATCH] Fix bug to prevent multiple updates of MAC on Beagleboard xM.

---
 drivers/net/usb/smsc95xx.c |   40 +++++++++++++++++++++++++++++++++++++---
 1 files changed, 37 insertions(+), 3 deletions(-)

diff --git a/drivers/net/usb/smsc95xx.c b/drivers/net/usb/smsc95xx.c
index 0c3c738..9f7673f 100644
--- a/drivers/net/usb/smsc95xx.c
+++ b/drivers/net/usb/smsc95xx.c
@@ -28,6 +28,7 @@
 #include <linux/usb.h>
 #include <linux/crc32.h>
 #include <linux/usb/usbnet.h>
+#include <asm/cputype.h>
 #include "smsc95xx.h"
 
 #define SMSC_CHIPNAME			"smsc95xx"
@@ -51,6 +52,7 @@ struct smsc95xx_priv {
 	spinlock_t mac_cr_lock;
 	bool use_tx_csum;
 	bool use_rx_csum;
+	bool set_rnd_mac;
 };
 
 struct usb_context {
@@ -654,6 +656,14 @@ static int smsc95xx_ioctl(struct net_device *netdev, struct ifreq *rq, int cmd)
 
 static void smsc95xx_init_mac_address(struct usbnet *dev)
 {
+	struct smsc95xx_priv *pdata = (struct smsc95xx_priv *)(dev->data[0]);
+	u32 i;
+	union {
+                u32 id;
+                u8 id_a[4];
+        } cpu;
+
+
 	/* try reading mac address from EEPROM */
 	if (smsc95xx_read_eeprom(dev, EEPROM_MAC_OFFSET, ETH_ALEN,
 			dev->net->dev_addr) == 0) {
@@ -665,10 +675,33 @@ static void smsc95xx_init_mac_address(struct usbnet *dev)
 		}
 	}
 
+#if defined (CONFIG_MACH_OMAP3_BEAGLE)
+	/* Ok, we have a Beagleboard, with no EEPROM, */
+	/* Create a MAC from the die id */
+
+	cpu.id = read_cpuid_id();
+
+	/* Now stuff the ballot...err..MAC box. */
+	cpu.id_a[2] &= 0xfe; /* clear multicast bit */
+	cpu.id_a[2] |= 0x02; /* set local assignment bit (IEEE802) */
+
+	for (i = 0; i < ETH_ALEN; i++) {
+		dev->net->dev_addr[i] = cpu.id_a[i];
+	}
+
+	/* OH YES, I DID THIS! Shoot me! */
+	return;
+#endif
+
 	/* no eeprom, or eeprom values are invalid. generate random MAC */
-	random_ether_addr(dev->net->dev_addr);
-	if (netif_msg_ifup(dev))
-		devdbg(dev, "MAC address set to random_ether_addr");
+	/* But only generate it once! */
+
+	if (!(pdata->set_rnd_mac)) {
+		random_ether_addr(dev->net->dev_addr);
+		if (netif_msg_ifup(dev))
+			devdbg(dev, "MAC address set to random_ether_addr");
+		pdata->set_rnd_mac = true;
+	}
 }
 
 static int smsc95xx_set_mac_address(struct usbnet *dev)
@@ -1051,6 +1084,7 @@ static int smsc95xx_bind(struct usbnet *dev, struct usb_interface *intf)
 
 	pdata->use_tx_csum = DEFAULT_TX_CSUM_ENABLE;
 	pdata->use_rx_csum = DEFAULT_RX_CSUM_ENABLE;
+	pdata->set_rnd_mac = false;
 
 	/* Init all registers */
 	ret = smsc95xx_reset(dev);
-- 
1.7.0.4

